"use strict";
// This file is part of phosphor-float-area, copyright (C) 2017 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var messaging_1 = require("@phosphor/messaging");
var widgets_1 = require("@phosphor/widgets");
var DialogLayout_1 = require("./DialogLayout");
var DialogMoveMessage = /** @class */ (function (_super) {
    __extends(DialogMoveMessage, _super);
    function DialogMoveMessage(widget, x, y, width, height) {
        var _this = _super.call(this, 'dialog-move') || this;
        _this.widget = widget;
        _this.x = x;
        _this.y = y;
        _this.width = width;
        _this.height = height;
        return _this;
    }
    DialogMoveMessage.prototype.conflate = function (other) {
        if (this.widget != other.widget)
            return (false);
        this.x = other.x;
        this.y = other.y;
        this.width = other.width;
        this.height = other.height;
        console.log(this.x + ' ' + other.x);
        return (true);
    };
    return DialogMoveMessage;
}(messaging_1.ConflatableMessage));
exports.DialogMoveMessage = DialogMoveMessage;
var DialogRaiseMessage = /** @class */ (function (_super) {
    __extends(DialogRaiseMessage, _super);
    function DialogRaiseMessage(widget, event) {
        var _this = _super.call(this, 'dialog-raise') || this;
        _this.widget = widget;
        _this.event = event;
        return _this;
    }
    return DialogRaiseMessage;
}(messaging_1.Message));
exports.DialogRaiseMessage = DialogRaiseMessage;
var Dialog = /** @class */ (function (_super) {
    __extends(Dialog, _super);
    function Dialog(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, { node: Dialog.createNode() }) || this;
        _this.addClass('charto-Dialog');
        _this.addClass('charto-Dialog-mod-dimmable');
        for (var _i = 0, _a = 'ns n,ew e,ns s,ew w,nwse nw,nesw ne,nesw sw,nwse se'.split(','); _i < _a.length; _i++) {
            var classList = _a[_i];
            var content = document.createElement('div');
            content.className = ('charto-Dialog-resize ' +
                classList.split(' ').map(function (resizer) { return 'charto-Dialog-resize-' + resizer; }).join(' '));
            _this.node.appendChild(content);
        }
        _this.layout = new DialogLayout_1.DialogLayout();
        return _this;
    }
    Dialog.createNode = function () {
        var node = document.createElement('div');
        return (node);
    };
    Dialog.prototype.onBeforeAttach = function (msg) {
        this.node.addEventListener('click', this);
        this.node.addEventListener('mousedown', this);
    };
    Dialog.prototype.onAfterDetach = function (msg) {
        this.node.removeEventListener('click', this);
        this.node.removeEventListener('mousedown', this);
    };
    Dialog.prototype.handleEvent = function (event) {
        var mouseEvent = event;
        switch (event.type) {
            case 'click':
                if (this.handleClick(mouseEvent))
                    break;
                return;
            case 'mousedown':
                if (this.handleMouseDown(mouseEvent))
                    break;
                return;
            case 'mousemove':
                this.handleMouseMove(mouseEvent);
                break;
            case 'mouseup':
                if (this.handleMouseUp(mouseEvent))
                    break;
                return;
        }
        event.preventDefault();
        event.stopPropagation();
    };
    Dialog.prototype.handleClick = function (event) {
        if (event.button != 0)
            return (false);
        messaging_1.MessageLoop.postMessage(this.parent, new DialogRaiseMessage(this, event));
        return (false);
    };
    Dialog.prototype.handleMouseDown = function (event) {
        if (event.button != 0)
            return (false);
        var moveX = 0;
        var moveY = 0;
        var resizeX = 0;
        var resizeY = 0;
        var target = event.target;
        if (target.parentNode == this.node) {
            var match = target.className.match(/charto-Dialog-resize-([ns]?)([ew]?)( |$)/);
            if (!match)
                return (false);
            // Resizing the dialog.
            if (match[1]) {
                if (match[1] == 'n') {
                    moveY = 1;
                    resizeY = -1;
                }
                else
                    resizeY = 1;
            }
            if (match[2]) {
                if (match[2] == 'w') {
                    moveX = 1;
                    resizeX = -1;
                }
                else
                    resizeX = 1;
            }
        }
        else if (target.className == 'p-TabBar-content' &&
            target.parentNode.parentNode.parentNode == this.node) {
            // Moving the dialog.
            moveX = 1;
            moveY = 1;
        }
        else
            return (false);
        this.removeClass('charto-Dialog-mod-dimmable');
        document.addEventListener('mousemove', this, true);
        document.addEventListener('mouseup', this, true);
        document.addEventListener('keydown', this, true);
        var node = this.node;
        this.drag = {
            moveX: moveX, moveY: moveY, resizeX: resizeX, resizeY: resizeY,
            x1: node.offsetLeft - event.clientX * moveX,
            y1: node.offsetTop - event.clientY * moveY,
            w1: node.offsetWidth - event.clientX * resizeX,
            h1: node.offsetHeight - event.clientY * resizeY
        };
        return (true);
    };
    Dialog.prototype.handleMouseMove = function (event) {
        var drag = this.drag;
        if (!drag)
            return;
        messaging_1.MessageLoop.postMessage(this.parent, new DialogMoveMessage(this, drag.x1 + event.clientX * drag.moveX, drag.y1 + event.clientY * drag.moveY, drag.w1 + event.clientX * drag.resizeX, drag.h1 + event.clientY * drag.resizeY));
    };
    Dialog.prototype.handleMouseUp = function (event) {
        if (event.button != 0)
            return (false);
        this.addClass('charto-Dialog-mod-dimmable');
        document.removeEventListener('mousemove', this, true);
        document.removeEventListener('mouseup', this, true);
        document.removeEventListener('keydown', this, true);
        this.drag = null;
        return (true);
    };
    Dialog.prototype.addWidget = function (widget, options) {
        if (options === void 0) { options = {}; }
        this.layout.addWidget(widget, options);
    };
    return Dialog;
}(widgets_1.Widget));
exports.Dialog = Dialog;
//# sourceMappingURL=Dialog.js.map