"use strict";
// This file is part of phosphor-float-area, copyright (C) 2017 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var domutils_1 = require("@phosphor/domutils");
var widgets_1 = require("@phosphor/widgets");
var Dialog_1 = require("./Dialog");
var FloatLayout_1 = require("./FloatLayout");
var EDGE_SIZE = 40;
var FloatArea = /** @class */ (function (_super) {
    __extends(FloatArea, _super);
    function FloatArea(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, { node: FloatArea.createNode() }) || this;
        _this.addClass('charto-FloatArea');
        if (options.overlay) {
            // Re-use an existing transparent overlay.
            // Pass it to a parent DockPanel first.
            _this.overlay = options.overlay;
            _this.overlayParent = _this.overlay.node.parentNode;
            _this.ownOverlay = false;
        }
        else {
            // Create a new transparent overlay inside this widget.
            _this.overlay = new widgets_1.DockPanel.Overlay();
            _this.overlay.node.classList.add('charto-mod-noTransition');
            _this.node.appendChild(_this.overlay.node);
            _this.overlayParent = _this.node;
            _this.ownOverlay = true;
        }
        var parentBox = domutils_1.ElementExt.boxSizing(_this.overlayParent);
        _this.edgeWidth = parentBox.borderLeft + parentBox.borderRight;
        _this.edgeHeight = parentBox.borderTop + parentBox.borderBottom;
        _this.layout = new FloatLayout_1.FloatLayout();
        return _this;
    }
    FloatArea.createNode = function () {
        var node = document.createElement('div');
        return (node);
    };
    FloatArea.prototype.onBeforeAttach = function (msg) {
        this.node.addEventListener('p-dragenter', this);
        this.node.addEventListener('p-dragleave', this);
        this.node.addEventListener('p-dragover', this);
        this.node.addEventListener('p-drop', this);
    };
    FloatArea.prototype.onAfterDetach = function (msg) {
        this.node.removeEventListener('p-dragenter', this);
        this.node.removeEventListener('p-dragleave', this);
        this.node.removeEventListener('p-dragover', this);
        this.node.removeEventListener('p-drop', this);
    };
    FloatArea.prototype.processMessage = function (msg) {
        switch (msg.type) {
            case 'dialog-move':
                var move = msg;
                this.layout.updateWidget(move.widget, move.x, move.y, move.width, move.height);
                break;
            case 'dialog-raise':
                var raise = msg;
                this.layout.raiseWidget(raise.widget, raise.event);
                break;
            default:
                _super.prototype.processMessage.call(this, msg);
        }
    };
    FloatArea.prototype.handleEvent = function (event) {
        switch (event.type) {
            case 'p-dragenter':
                if (this.handleDragEnter(event))
                    break;
                return;
            case 'p-dragleave':
                this.handleDragLeave(event);
                // Allow dragleave events to bubble up so overlay's parent
                // can see if it's time to hide it.
                return;
            case 'p-dragover':
                if (this.handleDragOver(event))
                    break;
                return;
            case 'p-drop':
                if (this.handleDrop(event))
                    break;
                return;
        }
        // Note: p-dragenter must be eaten to receive other drag events.
        event.preventDefault();
        event.stopPropagation();
    };
    FloatArea.prototype.handleDragEnter = function (event) {
        var widget = this.getDragged(event);
        if (!widget)
            return (false);
        var imageOffsetX = 0;
        var imageOffsetY = 0;
        var imageHeight = 0;
        // Equivalent to (dockPanel as any)._drag.dragImage if we had access.
        var dragImage = document.body.querySelector('.p-mod-drag-image');
        if (dragImage) {
            var imageRect = dragImage.getBoundingClientRect();
            imageOffsetX = dragImage.offsetLeft - imageRect.left;
            imageOffsetY = dragImage.offsetTop - imageRect.top;
            imageHeight = dragImage.offsetHeight;
        }
        var rect = this.node.getBoundingClientRect();
        var parentRect = this.overlayParent.getBoundingClientRect();
        var width = widget.node.offsetWidth;
        var height = widget.node.offsetHeight;
        var goldenRatio = 0.618;
        var inDialog = false;
        for (var parent_1 = widget.parent; parent_1; parent_1 = parent_1.parent) {
            if (parent_1 instanceof Dialog_1.Dialog) {
                inDialog = true;
                break;
            }
        }
        if (!inDialog) {
            // Widget is not inside a dialog, so it's probably docked.
            // Likely only one dimension was set by the user,
            // so make the proportions match the golden ratio.
            if (width > height / goldenRatio)
                width = height / goldenRatio;
            else if (height > width * goldenRatio)
                height = width * goldenRatio;
        }
        // Restrict initial floating panel size so its longer dimension
        // is half that of the area it's floating over.
        if (width > rect.width / 2) {
            width = rect.width / 2;
            height = width * goldenRatio;
        }
        if (height > rect.height / 2) {
            height = rect.height / 2;
            width = height / goldenRatio;
        }
        // Round size to integer.
        width = ~~(width + 0.5);
        height = ~~(height + 0.5);
        this.drag = {
            rect: rect,
            width: width,
            height: height,
            imageOffsetX: imageOffsetX,
            imageOffsetY: imageOffsetY,
            offsetLeft: parentRect.left + imageOffsetX,
            offsetTop: parentRect.top + imageOffsetY - imageHeight,
            offsetRight: parentRect.width - width - this.edgeWidth,
            offsetBottom: parentRect.height - height - this.edgeHeight
        };
        this.overlayVisible = false;
        this.handleDragOver(event);
        return (true);
    };
    FloatArea.prototype.handleDragLeave = function (event) {
        var related = event.relatedTarget;
        if (!related || !this.node.contains(related)) {
            // Mouse left the bounds of this widget.
            this.hideOverlay(event);
            this.drag = null;
        }
    };
    FloatArea.prototype.handleDragOver = function (event) {
        var drag = this.drag;
        if (!drag)
            return (false);
        if (this.onEdge(event)) {
            this.hideOverlay(event);
            return (false);
        }
        else {
            this.showOverlay(event);
        }
        var left = event.clientX - drag.offsetLeft;
        var top = event.clientY - drag.offsetTop;
        this.overlay.show({
            left: left, top: top,
            right: drag.offsetRight - left,
            bottom: drag.offsetBottom - top
        });
        // Tentatively accept the drag.
        event.dropAction = event.proposedAction;
        return (true);
    };
    FloatArea.prototype.handleDrop = function (event) {
        var _this = this;
        this.overlay.hide(0);
        if (!this.ownOverlay) {
            // Enable animated transitions in overlay movement.
            this.overlay.node.classList.remove('charto-mod-noTransition');
        }
        var drag = this.drag;
        if (!drag)
            return (false);
        // Let a parent dock panel handle drops near area edges.
        if (this.onEdge(event))
            return (false);
        var widget = this.getDragged(event);
        if (!widget) {
            event.dropAction = 'none';
            return (false);
        }
        // Deparent the widget and wait for layout changes to settle.
        widget.parent = null;
        this.layout.afterUpdate(function () {
            // Get updated float area bounds.
            var rect = _this.node.getBoundingClientRect();
            // Take ownership of the dragged widget.
            _this.addWidget(widget, {
                left: event.clientX - rect.left - drag.imageOffsetX,
                top: event.clientY - rect.top - drag.imageOffsetY,
                width: drag.width,
                height: drag.height
            });
        });
        this.update();
        // Accept the drag.
        event.dropAction = event.proposedAction;
        return (true);
    };
    FloatArea.prototype.getDragged = function (event) {
        // Only handle drag events containing widgets.
        if (!event.mimeData.hasData('application/vnd.phosphor.widget-factory'))
            return (null);
        var factory = event.mimeData.getData('application/vnd.phosphor.widget-factory');
        var widget = (typeof (factory) == 'function' && factory());
        // Ensure the dragged widget is known and is not a parent of this widget.
        if (!(widget instanceof widgets_1.Widget) || widget.contains(this))
            return (null);
        return (widget);
    };
    FloatArea.prototype.onEdge = function (event) {
        var rect = this.drag.rect;
        return (event.clientX - rect.left < EDGE_SIZE ||
            event.clientY - rect.top < EDGE_SIZE ||
            rect.right - event.clientX < EDGE_SIZE ||
            rect.bottom - event.clientY < EDGE_SIZE);
    };
    FloatArea.prototype.showOverlay = function (event) {
        if (this.overlayVisible)
            return;
        this.overlayVisible = true;
        if (this.ownOverlay) {
            if (this.node.parentNode) {
                // In case a parent DockPanel is also showing an overlay,
                // send a p-dragleave event to trigger hiding it.
                FloatLayout_1.sendLeaveEvent(event, this.node.parentNode);
            }
        }
        else {
            // Probably re-using a DockPanel's overlay,
            // so disable animated transitions in its movement.
            this.overlay.node.classList.add('charto-mod-noTransition');
        }
    };
    FloatArea.prototype.hideOverlay = function (event) {
        if (!this.overlayVisible)
            return;
        this.overlayVisible = false;
        if (this.ownOverlay) {
            this.overlay.hide(0);
        }
        else {
            // Enable animated transitions in overlay movement.
            this.overlay.node.classList.remove('charto-mod-noTransition');
        }
    };
    FloatArea.prototype.addWidget = function (widget, options) {
        if (options === void 0) { options = {}; }
        this.layout.addWidget(widget, options);
    };
    return FloatArea;
}(widgets_1.Widget));
exports.FloatArea = FloatArea;
//# sourceMappingURL=FloatArea.js.map