"use strict";
// This file is part of phosphor-float-area, copyright (C) 2017 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var messaging_1 = require("@phosphor/messaging");
var domutils_1 = require("@phosphor/domutils");
var widgets_1 = require("@phosphor/widgets");
var Dialog_1 = require("./Dialog");
var SimpleLayout_1 = require("./SimpleLayout");
var FloatLayoutItem = /** @class */ (function (_super) {
    __extends(FloatLayoutItem, _super);
    function FloatLayoutItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FloatLayoutItem.prototype.updateUser = function (x, y, width, height) {
        this.userX = x;
        this.userY = y;
        this.userWidth = width;
        this.userHeight = height;
    };
    return FloatLayoutItem;
}(widgets_1.LayoutItem));
exports.FloatLayoutItem = FloatLayoutItem;
var FloatLayout = /** @class */ (function (_super) {
    __extends(FloatLayout, _super);
    function FloatLayout(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, FloatLayoutItem) || this;
        _this.zTop = 0;
        return _this;
    }
    FloatLayout.prototype.addWidget = function (widget, options) {
        if (options === void 0) { options = {}; }
        var dialog = new Dialog_1.Dialog();
        var dockPanel = new widgets_1.DockPanel();
        dockPanel.addWidget(widget);
        dialog.addWidget(dockPanel);
        var handleEvent = dockPanel.handleEvent;
        var leaveEventSent = false;
        dockPanel.handleEvent = function (event) {
            switch (event.type) {
                case 'p-dragover':
                    if (!leaveEventSent && this.node.parentNode) {
                        // In case a parent DockPanel is also showing an overlay,
                        // send a p-dragleave event to trigger hiding it.
                        sendLeaveEvent(event, this.node.parentNode);
                        leaveEventSent = true;
                    }
                    break;
                case 'p-dragleave':
                    leaveEventSent = false;
                    break;
            }
            if (handleEvent)
                handleEvent.apply(this, arguments);
        };
        dockPanel.parent = dialog;
        dialog.parent = this.parent;
        messaging_1.MessageLoop.installMessageHook(dockPanel, function (handler, msg) {
            if (msg.type == 'child-removed' && msg.child instanceof widgets_1.TabBar) {
                // Allow the panel to process the message first.
                setTimeout(function () {
                    if (dockPanel.isEmpty)
                        dialog.close();
                    // TODO: dispose?
                }, 1);
            }
            // Let the message through.
            return (true);
        });
        var layoutItem = _super.prototype.addWidget.call(this, dialog);
        dialog.node.style.zIndex = '' + this.zTop;
        this.activeWidget = dialog;
        var box = domutils_1.ElementExt.boxSizing(dialog.node);
        var tabBar = (dockPanel.node.querySelector('.p-TabBar') || {});
        if (layoutItem) {
            this.updateItem(layoutItem, (options.left || 0) - box.paddingLeft - box.borderLeft, (options.top || 0) - box.paddingTop - box.borderTop, (options.width || 320) + box.horizontalSum, (options.height || 240) + box.verticalSum + (tabBar.offsetHeight || 0));
        }
        return (layoutItem);
    };
    FloatLayout.prototype.removeWidget = function (widget) {
        _super.prototype.removeWidget.call(this, widget);
    };
    FloatLayout.prototype.onUpdate = function () {
        var _this = this;
        var box = this.box;
        // Resize content to match the dialog.
        this.itemMap.forEach(function (item) { return _this.updateItem(item, item.userX, item.userY, item.userWidth, item.userHeight); });
    };
    FloatLayout.prototype.updateWidget = function (widget, x, y, width, height) {
        var item = this.itemMap.get(widget);
        if (item)
            this.updateItem(item, x, y, width, height);
    };
    FloatLayout.prototype.updateItem = function (item, x, y, width, height) {
        var box = this.box;
        item.updateUser(x, y, width, height);
        if (x < box.x)
            x = box.x;
        if (y < box.y)
            y = box.y;
        width = Math.max(Math.min(width, item.maxWidth || Infinity), item.minWidth);
        height = Math.max(Math.min(height, item.maxHeight || Infinity), item.minHeight);
        if (width > box.innerWidth)
            width = box.innerWidth;
        if (height > box.innerHeight)
            height = box.innerHeight;
        if (x - box.x + width > box.innerWidth)
            x = box.x + box.innerWidth - width;
        if (y - box.y + height > box.innerHeight)
            y = box.y + box.innerHeight - height;
        item.update(x, y, width, height);
    };
    FloatLayout.prototype.raiseWidget = function (widget, event) {
        if (widget != this.activeWidget) {
            widget.node.style.zIndex = '' + (++this.zTop);
            this.activeWidget = widget;
        }
    };
    FloatLayout.prototype.onFitRequest = function (msg) {
        // TODO: Calculate required size to fit children.
        // See DockLayout._fit
        _super.prototype.onFitRequest.call(this, msg);
    };
    return FloatLayout;
}(SimpleLayout_1.SimpleLayout));
exports.FloatLayout = FloatLayout;
/** Dispatch a new p-dragleave event outside any widgets. */
function sendLeaveEvent(event, node) {
    var leaveEvent = document.createEvent('MouseEvent');
    var oob = -1000;
    // Claim that the mouse entered the document body at faraway coordinates,
    // so any event receivers will consider it outside their bounds.
    leaveEvent.initMouseEvent('p-dragleave', true, true, window, 0, oob, oob, oob, oob, event.ctrlKey, event.altKey, event.shiftKey, event.metaKey, event.button, document.body);
    node.dispatchEvent(leaveEvent);
}
exports.sendLeaveEvent = sendLeaveEvent;
//# sourceMappingURL=FloatLayout.js.map