"use strict";
// This file is part of phosphor-float-area, copyright (C) 2017 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var messaging_1 = require("@phosphor/messaging");
var algorithm_1 = require("@phosphor/algorithm");
var domutils_1 = require("@phosphor/domutils");
var widgets_1 = require("@phosphor/widgets");
var SimpleLayout = /** @class */ (function (_super) {
    __extends(SimpleLayout, _super);
    function SimpleLayout(Item) {
        if (Item === void 0) { Item = widgets_1.LayoutItem; }
        var _this = _super.call(this) || this;
        _this.Item = Item;
        _this.box = { x: 0, y: 0, innerWidth: 0, innerHeight: 0, outerWidth: 0, outerHeight: 0 };
        _this.widgetList = [];
        _this.itemMap = new Map();
        _this.afterUpdateList = [];
        _this.isUpdated = true;
        return _this;
    }
    SimpleLayout.prototype.iter = function () {
        return algorithm_1.iter(this.widgetList);
    };
    SimpleLayout.prototype.dispose = function () {
        this.itemMap.forEach(function (item) { return item.dispose(); });
        this.itemMap.clear();
        for (var _i = 0, _a = this.widgetList; _i < _a.length; _i++) {
            var widget = _a[_i];
            widget.dispose();
        }
        _super.prototype.dispose.call(this);
    };
    SimpleLayout.prototype.afterUpdate = function (handler) {
        this.afterUpdateList.push(handler);
    };
    SimpleLayout.prototype.addWidget = function (widget) {
        var item;
        this.widgetList.push(widget);
        if (this.parent) {
            item = this.attachWidget(widget);
            this.parent.fit();
        }
        return (item);
    };
    SimpleLayout.prototype.removeWidget = function (widget) {
        this.widgetList.splice(this.widgetList.indexOf(widget), 1);
        if (this.parent) {
            this.detachWidget(widget);
            this.parent.fit();
        }
    };
    SimpleLayout.prototype.attachWidget = function (widget) {
        if (this.parent.node == widget.node.parentNode)
            return;
        var parentAttached = this.parent.isAttached;
        if (parentAttached)
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.BeforeAttach);
        this.parent.node.appendChild(widget.node);
        if (parentAttached)
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.AfterAttach);
        var item = new this.Item(widget);
        this.itemMap.set(widget, item);
        return (item);
    };
    SimpleLayout.prototype.detachWidget = function (widget) {
        if (this.parent.node != widget.node.parentNode)
            return;
        var parentAttached = this.parent.isAttached;
        if (parentAttached)
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.BeforeDetach);
        this.parent.node.removeChild(widget.node);
        if (parentAttached)
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.AfterDetach);
        var item = this.itemMap.get(widget);
        if (item) {
            this.itemMap.delete(widget);
            item.dispose();
        }
    };
    SimpleLayout.prototype.onBeforeAttach = function (msg) {
        _super.prototype.onBeforeAttach.call(this, msg);
        this.parent.fit();
    };
    SimpleLayout.prototype.onFitRequest = function (msg) {
        if (!this.parent.isAttached)
            return;
        var minWidth = 0;
        var minHeight = 0;
        var maxWidth = Infinity;
        var maxHeight = Infinity;
        this.itemMap.forEach(function (item) {
            item.fit();
            minWidth = Math.max(minWidth, item.minWidth);
            minHeight = Math.max(minHeight, item.minHeight);
            maxWidth = Math.min(maxWidth, item.maxWidth);
            maxHeight = Math.min(maxHeight, item.maxHeight);
        });
        this.updateBox();
        var extraWidth = this.box.outerWidth - this.box.innerWidth;
        var extraHeight = this.box.outerHeight - this.box.innerHeight;
        var style = this.parent.node.style;
        minWidth += extraWidth;
        minHeight += extraHeight;
        maxWidth += extraWidth;
        maxHeight += extraHeight;
        style.minWidth = minWidth + 'px';
        style.minHeight = minHeight + 'px';
        style.maxWidth = maxWidth + 'px';
        style.maxHeight = maxHeight + 'px';
        this.isUpdated = false;
        if (this.parent.parent) {
            messaging_1.MessageLoop.sendMessage(this.parent.parent, widgets_1.Widget.Msg.FitRequest);
        }
        if (!this.isUpdated) {
            messaging_1.MessageLoop.sendMessage(this.parent, widgets_1.Widget.Msg.UpdateRequest);
        }
    };
    SimpleLayout.prototype.onUpdateRequest = function (msg) {
        if (this.parent.isVisible)
            this.update();
    };
    SimpleLayout.prototype.onResize = function (msg) {
        if (this.parent.isVisible)
            this.update(msg.width, msg.height);
    };
    SimpleLayout.prototype.updateBox = function (width, height) {
        if (width === void 0) { width = this.parent.node.offsetWidth; }
        if (height === void 0) { height = this.parent.node.offsetHeight; }
        var box = this.box;
        var sizing = domutils_1.ElementExt.boxSizing(this.parent.node);
        box.x = sizing.paddingLeft,
            box.y = sizing.paddingTop,
            box.innerWidth = width - sizing.horizontalSum;
        box.innerHeight = height - sizing.verticalSum;
        box.outerWidth = width;
        box.outerHeight = height;
    };
    SimpleLayout.prototype.update = function (width, height) {
        this.isUpdated = true;
        this.updateBox(width, height);
        this.onUpdate();
        if (this.afterUpdateList.length) {
            for (var _i = 0, _a = this.afterUpdateList; _i < _a.length; _i++) {
                var handler = _a[_i];
                handler();
            }
            this.afterUpdateList = [];
        }
    };
    SimpleLayout.prototype.onUpdate = function () { };
    SimpleLayout.prototype.updateItem = function (item, x, y, width, height) {
        item.update(x, y, width, height);
    };
    return SimpleLayout;
}(widgets_1.Layout));
exports.SimpleLayout = SimpleLayout;
//# sourceMappingURL=SimpleLayout.js.map